;********************************************************************************
;*  ROV Controller (RX)								*
;*										*
;*  Version 1.3									*
;*  by Eightball Engineering							*
;*  with assistance from Nautilus Robotics					*
;*  2001.6.15									*
;*										*
;*  Revision History:								*
;*	June 15/01 GP - PWM algorithm implemented for DC motors.		*
;*	June 16/01 AK -	Conversion algirithm implemented for joystick coordinate*
;*			mapping. Now forward, reverse, left and right motion	*
;*			is corrected for submarine propultion.			*
;*	June 17/01 GP - Two byte serial communication implemented.		*
;*	June 22/01 GP - Depth motor PWM added.					*
;*	June 22/01 AK - Fix position/speed jump table to fix control while	*
;*			backing up.						*
;*	June 23/01 GP - Servo control subroutine added.				*
;*	June 23/02 GP - DC motor soft start algorithm.				*
;*	July 06/02 GP - Microcontroller status LED.				*
;*	July 10/02 GP - Servo control subroutine removed			*
;*	July 10/02 GP - Water sensor input implemented.				*
;*	July 11/02 GP - One Wire serial implemented.				*
;*	July 12/01 GP - Implemented I^2C communication protocol.		*
;*	July 21/01 GP - Fixed communications bug.				*
;*										*
;*  The micro receives control instructions from the joystick micro via serial 	*
;*  connection. 16 bits with start and stop bit are sent from the joystick	*
;*  micro when it receives an acknowlege. The joystick micro continuously	*
;*  transmitts with a delay between each data packet.				*
;*  										*
;*  PWM algorithms are used to drive DC permanent magnet motors via an H-bridge	*
;*  motor driver ic at various speeds.						*
;*										*
;*  The micro controls three DC motors; a left and right motor and third to	*
;*  control the depth of the submarine. An output for the control of a		*
;*  light is also provided. A water sensor input alows the micro to surface	*
;*  the ROV and shut down in case of water leaking into the electronics 	*
;*  compartment; this will prevent a malfuntion/loss of control.		*
;*										*
;*										*
;*  VALUE_1 register:								*
;*										*
;*  ___7______6______5______4______3______2______1______0__			*
;* |      |      |      |      |      |      |      |      | 			*
;*  _______________________________________________________			*
;*   LIGHT SERVO   ----LEFT MOTOR----	---RIGHT MOTOR----			*
;********************************************************************************


;********************************************************************************
;* Processor definition and configuration					*
;********************************************************************************
; 	Clock = 4MHz external resonator

  	LIST P=16F84
  	#include "P16F84.inc"

	__CONFIG _CP_OFF & _WDT_OFF & _XT_OSC & _PWRTE_ON

	ERRORLEVEL -302		; turn off bank bits warning
  
;********************************************************************************
;* Hardware declaration								*
;********************************************************************************

;#define ACK	PORTA, 1	; (pin 18) acknowledge output
;#define SER_IN	PORTA, 0	; (pin 17) serial data input

#define SER_IN	PORTA, 1	; (pin 18) serial data input

#define RED	PORTA, 3	; (pin 2) system Check output
#define PROBE	PORTB, 5	; (pin 11) water probe input
#define LIGHT	PORTB, 4	; (pin 10) relay output for light

#define MOTRCW	PORTB, 2	; (pin 8) motor Right motor CW output
#define MOTRCCW	PORTB, 3	; (pin 9) motor Right motor CCW output
#define MOTLCCW	PORTB, 6	; (pin 12) motor Left motor CW output
#define MOTLCW	PORTB, 7	; (pin 13) motor Left motor CCW output
#define MOTDCCW	PORTB, 0	; (pin 6) motor D CW output
#define MOTDCW	PORTB, 1	; (pin 7) motor D CCW output


;********************************************************************************
;* Constants Declaration							*
;********************************************************************************
; the duty cycle of the DC motors ranges from 0% to 100% and is represented by
; 0 to 200

STOP	equ	.0		; motor stopped, 0% duty cycle
SLOW	equ	.75		; motor slow speed, 25% duty cycle
MED	equ	.150		; motor medium speed, 50% duty cycle
FAST	equ	.200		; motor fast speed, 100% duty cycle

; the rate of change of duty cycle

UP	equ	.25		; the rate at which duty goes up
DOWN	equ	.50		; the rate at which duty does down

; loop counter for status LED blink rate

RATE	equ	.4		; main loop is 40Hz, blink rate is 40/RATE


;********************************************************************************
;* Variable declaration								*
;********************************************************************************

	cblock	0x0C

	L_DUTY			; L motor duty cycle
	R_DUTY			; R motor duty cycle
	D_DUTY			; R motor duty cycle
	TEMP			; temporary variable
	TEMP_L			; temporary duty cyle variable
	TEMP_R			; temporary duty cyle variable
	TEMP_D			; temporary duty cyle variable
	TEMPY			; temporary servo position
	CNTR			; counter variable
	CNTR_1			; counter variable
	CNTR_2			; counter variable
	VALUE_1			; input first serial byte
	VALUE_2			; input second serial byte
	NOISE			; input serial noise flag
	COUNT			; loop counter
	RATE_CNTR		; blink rate counter of status LED

	endc

;********************************************************************************
;* Start of Program								*
;********************************************************************************

	org	0x00			
	goto	INIT		; reset vector

	org	0x04
	goto	INTR		; interrupt vector


;********************************************************************************
;* Microcontroller Initialization						*
;********************************************************************************

INIT	clrf	PORTA		; initialize PortA		
	bsf	STATUS, RP0	; Select Bank 1
;	movlw	0x15		; RA0, RA2 & RA4 input, RA1 & RA3 outputs
;	movwf	TRISA
	movlw	0x17		;  RA0, RA1, RA2 & RA4 input, RA3 output
	movwf	TRISA

	bcf	STATUS, RP0	; return to Bank0
	
	clrf	PORTB		; initialize PortB		
	bsf	STATUS, RP0	; Select Bank 1
	movlw	0x20		; RB5 input, the rest output
	movwf	TRISB		
	movlw	0xC0		; internal pullups disabled
	movwf	OPTION_REG
	bcf	STATUS, RP0	; return to Bank0

	clrf	INTCON		; disable interupts

	clrf	VALUE_1		; clear serial input data
	clrf	VALUE_2		; clear serial input data

	clrf	L_DUTY		; start with left motor off
	clrf	R_DUTY		; start with right motor off
	clrf	D_DUTY		; start with depth motor off

	movlw	RATE
	movwf	RATE_CNTR	; blink rate of status indicator LED


;********************************************************************************
;* Main Program Loop								*
;********************************************************************************

MAIN	bcf	RED		; turn status LED off
	call	BLINK		; turn status LED on (determined by RATE)
	call	WATER		; check for leaks in electronics canister
	call	GETDATA		; retrieve new data in VALUE_1 register
	call	BUTTON		; service button data before messing with VALUE_1

	movf	VALUE_1, W	; VALUE_1 contains joystick position info (0 to 64)
	andlw	0x3F		; mask out button bits
	call	POSJUMP		; convert into speed and direction
	movwf	VALUE_1		; update VALUE_1 register

	call	GET_L		; retrieve left motor dutycycle data (in W)
	call	TABLE		; convert data to dutycycle (in W)
	movwf	TEMP_L		; TEMP_L is duty wanted
	btfsc	STATUS, Z	; if duty wanted is zero then
	clrf	L_DUTY		; clear duty cycle
	btfss	STATUS, Z	; if duty wanted is not zero then
	call	RAMP_L		; ramp duty cycle up/down
	
	call	GET_R		; retrieve right motor dutycycle data (in W)
	call	TABLE		; convert data to dutycycle (in W)
	movwf	TEMP_R		; TEMP_R is duty wanted
	btfsc	STATUS, Z	; if duty wanted is zero then
	clrf	R_DUTY		; clear duty cycle
	btfss	STATUS, Z	; if duty wanted is not zero then
	call	RAMP_R		; ramp duty cycle up/down

	call	GET_D		; retrieve depth motor dutycycle data (in W)

	call	PWM		; PWM output to motors
	call	PWM		; PWM is called numerous times (without getting
	call	PWM		; new data serially) with the same duty cycle
	call	PWM		; this allows a duty cycle
	call	PWM		; closer to 100%
	call	PWM
	call	PWM
	call	PWM
	call	PWM
	call	PWM

	goto	MAIN


;********************************************************************************
;* Motor PWM Dytycycle Table							*
;********************************************************************************
; table returns a VALUE_1 for the dutycycle for the motors, the dutycycle is a
; number from 0 to 255, 0 being 0% and 255 equivalent to 100% duty.

TABLE  	addwf  PCL, F
 	retlw  STOP		; 0% Duty
 	retlw  SLOW		; 25% Duty
 	retlw  MED		; 50% Duty
  	retlw  FAST		; 100% Duty


;********************************************************************************
;* POSJUMP -- Joystick Data to Motor Speed Jump Table				*
;********************************************************************************
; Uses first six bits of received joystick position data X and Y (64 unique
; positions) to reference and return a left and right motor direction/speed
; combination as:
;  ___7________6________5________4________3________2________1________0___
; |_______||_______||_______||_______||_______||_______||_______||_______|
;     0        0     \DIR_L/  \-- SPEED_L ---/  \DIR_R/  \-- SPEED_R ---/
;

; Note that the joystick position grid contains a one position dead-zone about
; each axis.  Example: X=011,Y=000 is identical speedwise as X=011,Y=100 (in
; both cases speed Y is zero).


; DEBUG COMMENT: Speed settings adjacent to X position axis may need
;		 tuning as direction of travel during turning may be reversed. 


POSJUMP	addwf	PCL,F			; Jump program counter ahead to position index
	
	retlw	b'00000011'		; X=000 Y=000 -> (bank left) left stop,	right full ahead
	retlw	b'00101010'		; X=000 Y=001 -> left reverse slow,	right ahead med
	retlw	b'00110001'		; X=000 Y=002 -> left reverse med,	right ahead slow
	retlw	b'00111011'		; etc, etc    -> left reverse full,	right ahead full  (left full turn)
	retlw	b'00111011'		; -> as above
	retlw	b'00101010'		; -> left reverse slow,	right reverse fast
	retlw	b'00110001'		; -> left reverse med,	right reverse slow
	retlw	b'00111000'		; -> left reverse full,	right stop

	retlw	b'00001011'		; -> left ahead slow,	right ahead full
	retlw	b'00000010'		; -> left stop,		right ahead med
	retlw	b'00101001'		; -> left reverse slow,	right ahead slow
	retlw	b'00110001'		; -> left reverse med,	right ahead med  (left med turn)
	retlw	b'00110001'		; -> as above
	retlw	b'00101001'		; -> left reverse slow,	right ahead slow
	retlw	b'00110000'		; -> left reverse med,	right stop
	retlw	b'00111101'		; -> left reverse fast,	right reverse slow

	retlw	b'00010011'		; -> left forward med,	right ahead full
	retlw	b'00001010'		; -> left forward slow,	right ahead med
	retlw	b'00000001'		; -> left stop,		right ahead slow
	retlw	b'00101001'		; -> left reverse slow, right ahead slow  (left slow turn)
	retlw	b'00101001'		; -> as above
	retlw	b'00101000'		; -> left reverse slow,	right stop
	retlw	b'00110101'		; -> left reverse med,	right reverse slow
	retlw	b'00111110'		; -> left reverse full,	right reverse med

	retlw	b'00011011'		; -> forward full
	retlw	b'00010010'		; -> forward medium
	retlw	b'00001001'		; -> forward slow
	retlw	b'00000000'		; -> all stop
	retlw	b'00000000'		; -> all stop
	retlw	b'00101101'		; -> reverse slow
	retlw	b'00110110'		; -> reverse medium
	retlw	b'00111111'		; -> reverse full

	retlw	b'00011011'		; -> forward full
	retlw	b'00010010'		; -> forward medium
	retlw	b'00001001'		; -> forward slow
	retlw	b'00000000'		; -> all stop
	retlw	b'00000000'		; -> all stop
	retlw	b'00101101'		; -> reverse slow
	retlw	b'00110110'		; -> reverse medium
	retlw	b'00111111'		; -> reverse full

	retlw	b'00011010'		; -> left ahead full,	right ahead med
	retlw	b'00010001'		; -> left ahead med,	right ahead slow
	retlw	b'00001000'		; -> left ahead slow,	right stop
	retlw	b'00001101'		; -> left ahead slow,	right reverse slow  (right slow turn0
	retlw	b'00001101'		; -> as above
	retlw	b'00000101'		; -> left stop,		right reverse slow
	retlw	b'00101110'		; -> left reverse slow,	right reverse med
	retlw	b'00110111'		; -> left reverse med,	right reverse fast

	retlw	b'00011101'		; -> left ahead full,	right ahead slow
	retlw	b'00010000'		; -> left ahead med,	right stop
	retlw	b'00001101'		; -> left ahead slow,	right reverse slow
	retlw	b'00010110'		; -> left ahead med,	right reverse med  (right med turn)
	retlw	b'00010110'		; -> as above
	retlw	b'00001101'		; -> left ahead slow,	right reverse slow 
	retlw	b'00000110'		; -> left stop,		right reverse med
	retlw	b'00101111'		; -> left reverse slow,	right reverse fast

	retlw	b'00011000'		; -> left ahead full,	right stop
	retlw	b'00010101'		; -> left ahead med,	right reverse slow
	retlw	b'00001110'		; -> left ahead slow,	right reverse med
	retlw	b'00011111'		; -> left ahead full,	right reverse fast  (right full turn)
	retlw	b'00011111'		; -> as above
	retlw	b'00010101'		; -> left ahead med,	right reverse slow
	retlw	b'00001110'		; -> left ahead slow,	right reverse med
	retlw	b'00000111'		; -> left stop,		right reverse fast  


;********************************************************************************
;* Soft Start Algorithm for DC Motors						*
;********************************************************************************

RAMP_L	movf	TEMP_L, W	; retrieve desired duty cycle
	xorwf	L_DUTY, W	; is current duty equal to desired duty?
	btfsc	STATUS, Z

	return			; if yes then return, else increase/decrease duty
	
	movf	TEMP_L, W	; retrieve desired duty cycle		
	subwf	L_DUTY, W	; subtract desired duty from current duty
	btfsc	STATUS, C	; if carry = 1 => result is positive and desired < current
	call	DOWN_L		; then need to decrease current duty
	btfss	STATUS, C	; if carry = 0 => result is negative and desired > current
	call	UP_L		; then need to increase current duty

	return
	
RAMP_R	movf	TEMP_R, W	; retrieve desired duty cycle
	xorwf	R_DUTY, W	; is current duty equal to desired duty?
	btfsc	STATUS, Z

	return			; if yes then return, else increase/decrease duty
	
	movf	TEMP_R, W	; retrieve desired duty cycle		
	subwf	R_DUTY, W	; subtract desired duty from current duty
	btfsc	STATUS, C	; if carry = 1 => result is positive and desired < current
	call	DOWN_R		; then need to decrease current duty
	btfss	STATUS, C	; if carry = 0 => result is negative and desired > current
	call	UP_R		; then need to increase current duty

	return


DOWN_L	movlw	DOWN		; decrease duty
	clrf	L_DUTY
;	subwf	L_DUTY, F
	return

UP_L	movlw	UP		; increase duty
	addwf	L_DUTY, F
	return

DOWN_R	movlw	DOWN		; decrease duty
	clrf	R_DUTY
;	subwf	R_DUTY, F
	return

UP_R	movlw	UP		; increase duty
	addwf	R_DUTY, F
	return


;********************************************************************************
;* System Status Indication							*
;********************************************************************************
; during normal operation (looping trough MAIN) blink is called and status LED
; is blinked. When microprocessor is halted LED stops blinking.

BLINK	decfsz	RATE_CNTR, F
	return

	bsf	RED		; turn on LED
	movlw	RATE		; load W with blink rate
	movwf	RATE_CNTR	; store blink rate into rate counter
	return


;********************************************************************************
;* Automatic ROV Surfacing Sequence						*
;********************************************************************************
; the water probe input will go high if water is detected in the electronics
; compartment. When water is detected it is desireble to have the ROV surface
; immediately before the micro malfunctions.

WATER	btfsc	PROBE		; test for water in the electronics compartment
	goto	SURFACE		; if water is detected, surface the ROV	
	return			; else return

SURFACE bcf	MOTLCW		; turn off all motors
	bcf	MOTLCCW
	bcf	MOTRCW
	bcf	MOTRCCW
	bcf	MOTDCW
	bcf	MOTDCCW
	bcf	RED

	bsf	MOTDCW		; then set CW depth motor output
	bsf	LIGHT		; turn on light
	sleep			; shut down processor

;********************************************************************************
;* Button service routine							*
;********************************************************************************

BUTTON	btfsc	VALUE_1, 7	; check LIGHT button
	bsf	LIGHT		; if set then turn light on
	btfss	VALUE_1, 7	; else if not set
	bcf	LIGHT		; then turn light off
;	btfsc	VALUE_1, 6	; check SERVO button
;	goto	SERVO		; if set then joystick controls the servo

	return


;********************************************************************************
;* Dutycycle control								*
;********************************************************************************
; this loops 200 times, the duty cycle variable for the right and left motor
; could be from 0 to 200 At the start of the loop the duty cycle variable is
; checked, if is is zero then the output remains low, if the variable is not
; zero then the output pulse is set high. The variable is decremented each time
; through the loop. The output pulse is set high when the variable is zero
; A duty cycle variable of 200 would represent 100% and a duty cycle variable of
; 100 would represent 50% etc. 

PWM	movlw	.200
	movwf	COUNT

	movf	L_DUTY, W	; store in temporary register
	movwf	TEMP_L
	btfss	STATUS, Z	; if left duty cycle is NOT zero
	call	HIGH_L		; set output for CW or CCW control line

	movf	R_DUTY, W	; store in temporary register
	movwf	TEMP_R
	btfss	STATUS, Z	; if right duty cycle is NOT zero
	call	HIGH_R		; set output for CW or CCW control line

	movf	D_DUTY, W	; store in temporary register
	movwf	TEMP_D
	btfss	STATUS, Z	; if left duty cycle is NOT zero
	call	HIGH_D		; set output for CW or CCW control line

DUTY	decf	TEMP_L, F	; decrement duty cycle variable
	btfsc	STATUS, Z	; if dutycycle variable is zero
	call	LOW_L		; then clear CW or CCW output

	decf	TEMP_R, F	; decrement duty cycle variable
	btfsc	STATUS, Z	; if duty cycle variable is zero
	call	LOW_R		; then clear CW or CCW output

	decf	TEMP_D, F	; decrement duty cycle variable
	btfsc	STATUS, Z	; if dutycycle variable is zero
	call	LOW_D		; then clear CW or CCW output

	decfsz	COUNT, F	; loop counter
	goto	DUTY

	return


;********************************************************************************
;* Direction Detection of Left, Right and Depth Motor and Set of Control Line	*
;********************************************************************************

HIGH_L	btfss	VALUE_1, 5	; check direction of left motor
	bsf	MOTLCW		; then set clock wise motor output
	btfsc	VALUE_1, 5	; check direction of left motor
	bsf	MOTLCCW		; then set counter clock wise motor output

	return

HIGH_R	btfss	VALUE_1, 2	; check direction of left motor
	bsf	MOTRCW		; then set clock wise motor output
	btfsc	VALUE_1, 2	; check direction of left motor
	bsf	MOTRCCW		; then set counter clock wise motor output

	return

HIGH_D	btfss	VALUE_2, 6	; check direction of depth motor
	bsf	MOTDCW		; then set clock wise motor output
	btfsc	VALUE_2, 6	; check direction of depth motor
	bsf	MOTDCCW		; then set counter clock wise motor output

	return

;********************************************************************************
;* Direction Detection of Left and Right Motor and Clear of Control Line	*
;********************************************************************************
; this subroutine check the direction bits in the VALUE_1 register and clears
; the output for the corresponding direction output control line for the
; left and right motors

LOW_L	btfss	VALUE_1, 5	; check direction of left motor
	bcf	MOTLCW		; then clear clock wise motor output
	btfsc	VALUE_1, 5	; check direction of left motor
	bcf	MOTLCCW		; then clear counter clock wise motor output

	return

LOW_R	btfss	VALUE_1, 2	; check direction of left motor
	bcf	MOTRCW		; then clear clock wise motor output
	btfsc	VALUE_1, 2	; check direction of left motor
	bcf	MOTRCCW		; then clear counter clock wise motor output

	return

LOW_D	btfss	VALUE_2, 6	; check direction of depth motor
	bcf	MOTDCW		; then clear clock wise motor output
	btfsc	VALUE_2, 6	; check direction of depth motor
	bcf	MOTDCCW		; then clear counter clock wise motor output

	return

;********************************************************************************
;* Retrieve Left Motor Dutycycle variable from VALUE_1 Register			*
;********************************************************************************
; the VALUE_1 returned in W is a number from 0 to 3 for the X-axis
; this VALUE_1 is used in TABLE to return a dutycycle for the left motor, if direction
; is in the negative X direction then the VALUE_1 is complimented such that
; the VALUE_1 will increase for increased joystick motion (joystick is center zero)

GET_L	movf	VALUE_1, W	; retreive VALUE_1
	movwf	TEMP
	rrf	TEMP, F		; shift 3 times to get X-axis VALUE_1s in 
	rrf	TEMP, F		; LSB position
	rrf	TEMP, F
	movlw	0x03		; mask TEMP, only want X-axis VALUE_1
	andwf	TEMP, W		; now W contains the 2 bit VALUE_1 for X
	
	return


;********************************************************************************
;* Retrieve Right Motor Dutycycle variable from VALUE_1 Register		*
;********************************************************************************
; the VALUE_1 returned in W is a number from 0 to 3 for the Y-axis
; this VALUE_1 is used in TABLE to return a speed for the right motor, if direction
; is in the negative Y direction then the VALUE_1 is complimented such that
; the VALUE_1 will increase for increased joystick motion (joystick is center zero)

GET_R	movf	VALUE_1, W	; retreive VALUE_1
	movwf	TEMP
	movlw	0x03		; mask TEMP, only want Y-axis VALUE_1
	andwf	TEMP, W		; W contains the 2 bit VALUE_1 for Y-axis

	return
	

;********************************************************************************
;* Retrieve Depth Motor Dutycycle variable from VALUE_2 Register		*
;********************************************************************************
; the VALUE_2 register contains a number from 0 to 127 corresponding to the 
; position of the potentiometer that will control the depth motor's speed
; and direction. The potentiometer is centered at 63. The MSB is used to
; determine the direction of the motor since it will be set for numbers
; greater than 63 and not set for numbers lower than 63. The motor must
; increase speed for 63 - 127 and for 63 - 0, thus need to compliment
; if number is less than 63.

GET_D	movf	VALUE_2, W	; retreive VALUE_2
	movwf	D_DUTY
	btfss	VALUE_2, 6	; check direction, if bit is not set
	comf	D_DUTY, F	; then need to compliment
	movlw	.64		; subtract 64 so that duty cycle starts at
	subwf	D_DUTY, F	; 0%
	rlf	D_DUTY, F	; multiply by 4 so that duty goes up to 100%
	rlf	D_DUTY, F
	movlw	0xC0		; mask LSB six bits, want to check for higher
	andwf	D_DUTY, W	; than 64 (greater than 25% dutycycle)
	btfsc	STATUS, Z	; is the duty cycle greater than 25%?
	clrf	D_DUTY		; if not then set duty to 0%, else duty remains

	return


;********************************************************************************
;* Retrieve serial VALUE_1 and VALUE_2 subroutine				*
;********************************************************************************
; the transmitting micro transmitts at least 832us of silence. The minimum time
; is based on the time 16 data bits (52us/bit) take to transmit, worst case is
; when all data bits are low. Thus a minimum of 832us must be detected in order
; to know when to start polling for the start bit of serial data.

GETDATA	call	SYNCRO		; find the transmitted silence
	call	SERIAL		; receive serial VALUE_1 and VALUE_2
	movf	NOISE, F	; check for noise (movf effects zero flag)
	btfss	STATUS, Z	; if there is noise			DEBUG !!
	bsf	LIGHT		; turn light on				DEBUG !!

	btfss	STATUS, Z	; if there is noise
	goto	GETDATA		; then get get VALUE_1 and VALUE_2 again
				; else return with data in VALUE_1 and VALUE_2
	return


;********************************************************************************
;* Syncronizing Subroutine							*
;********************************************************************************
; syncro takes 12us to execute
; if silence is sampled 70 times through then 840us of silence must have
; occured and thus the next bit to be transmitted is the start bit.

SYNCRO	incf	CNTR, F
	btfss	SER_IN		; skip next instruction if silent
	clrf	CNTR		; something being transmitted, clear counter
	nop
	nop			; need to detect minimum of 832us of silence
	nop			; maximum time between sampling is 25us
	nop			; since each bit takes 52us to transmit
	movf	CNTR, W		
	xorlw	.70		; test if we have received 70 silent samples
	btfss	STATUS, Z
	goto	SYNCRO		; if not, try again

	return

;********************************************************************************
;* Serial Input subroutine							*
;********************************************************************************
; 2 bytes are sampled in sequence and stored in VALUE_1 and VALUE_2
; the start bit is logic 1, bits are read in from LSB to MSB
; 3 samples are taken in each bit, if noise is detected then the noise flag is set
; FIND takes 8 cycles, executed 6 times, 5 instructions at the end totals to
; 52us, 19.2K baud

SERIAL	clrf	CNTR		; clear counter before entering loop

FIND	incf	CNTR, F
	btfsc	SER_IN		; execute next instruction if start bit is received
	clrf	CNTR		; no TRUE sample was received - reset COUNT to 0.
	movf	CNTR, W
	xorlw	0x06		; test if we have received 6 TRUE samples since last FALSE.
	btfss	STATUS, Z
	goto	FIND
	
	movlw	.8		; loop counter, 8 bits sampled
	movwf	CNTR_1		; first byte
	movwf	CNTR_2		; second byte
	movlw	.16		; load NOISE with 16 for decrementing in each loop
	movwf	NOISE		; NOISE will contain 0 if no noise is detected			

LOOP1	clrf	COUNT		; LOOP samples 1 bit at a time, three samples per bit, 8 bits		
	nop			; each loop through must take 52us, 19.2K baud
	nop			; thus 52 instructions per loop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop			; extra nop to make 53us, patch
	nop
	nop
	nop
;	bsf	LIGHT		; DEBUG output for oscilloscope		
	btfss	SER_IN		; 1st sample
	incf	COUNT, F
;	bcf	LIGHT
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;	bsf	LIGHT		; DEBUG output for oscilloscope			
	btfss	SER_IN		; 2nd sample
	incf	COUNT, F
;	bcf	LIGHT	
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;	bsf	LIGHT		; DEBUG output for oscilloscope				
	btfss	SER_IN		; 3rd sample
	incf	COUNT, F
;	bcf	LIGHT
	nop
	nop
	rrf	VALUE_1, F	; shift in all 8 bits		
	movf	COUNT, W	; now use COUNT to detect noise
	
	andlw	0x03		; if all 3 samples are low
	btfsc	STATUS, Z
	bcf	VALUE_1, 7	; then bit = 0
	btfsc	STATUS, Z
	decf	NOISE, F	; and decrement NOISE

	xorlw	0x03		; if all 3 samples are high
	btfsc	STATUS, Z
	bsf	VALUE_1, 7	; then bit = 1
	btfsc	STATUS, Z
	decf	NOISE, F	; and decrement NOISE

	decfsz	CNTR_1, F	; decrement loop counter (8 bits)
	goto	LOOP1		; continue till all 8 bits are read in

				; second byte starts here

LOOP2	clrf	COUNT		; LOOP samples 1 bit at a time, three samples per bit, 8 bits		
	nop			; each loop through must take 52us, 19.2K baud
	nop			; thus 52 instructions per loop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop			; extra nop to make 53us, patch
	nop
	nop
	nop
;	bsf	LIGHT		; DEBUG output for oscilloscope	
	btfss	SER_IN		; 1st sample
;	btfsc	SER_IN		; 1st sample
	incf	COUNT, F
;	bcf	LIGHT		; DEBUG output for oscilloscope
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;	bsf	LIGHT		; DEBUG output for oscilloscope	
	btfss	SER_IN		; 2nd sample
	incf	COUNT, F
;	bcf	LIGHT		; DEBUG output for oscilloscope
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;	bsf	LIGHT		; DEBUG output for oscilloscope	
	btfss	SER_IN		; 3rd sample
	incf	COUNT, F
;	bcf	LIGHT		; DEBUG output for oscilloscope	
	nop
	nop
	rrf	VALUE_2, F	; shift in all 8 bits		
	movf	COUNT, W	; now use COUNT to detect noise
	
	andlw	0x03		; if all 3 samples are low
	btfsc	STATUS, Z
	bcf	VALUE_2, 7	; then bit = 0
	btfsc	STATUS, Z
	decf	NOISE, F	; and decrement NOISE

	xorlw	0x03		; if all 3 samples are high
	btfsc	STATUS, Z
	bsf	VALUE_2, 7	; then bit = 1
	btfsc	STATUS, Z
	decf	NOISE, F	; and decrement NOISE

	decfsz	CNTR_2, F	; decrement loop counter (8 bits)
	goto	LOOP2		; continue till all 8 bits are read in

	return


;********************************************************************************
;* Interrupt routine								*
;********************************************************************************

INTR	clrf	INTCON		; prevent recursive interrupt

	return


;********************************************************************************
;* Delay subroutine - W contains No. of milliseconds to delay			*
;********************************************************************************

DELAY	movwf	CNTR_2
	
LOOP_B	movlw	.200		; outer loop, executed CNTR2 times
	movwf	CNTR_1

LOOP_A	nop			; inner loop, executed CNTR1 times
	nop
	decfsz	CNTR_1,F
	goto	LOOP_A

	decfsz	CNTR_2,F		
	goto	LOOP_B	

	return


;********************************************************************************
;* End of program								*
;********************************************************************************

	end	

