;********************************************************************************
;*  Joystick Controller (TX)							*
;*  using PIC12C671								*
;*										*
;*  Version 1.3									*
;*  by Eightball Engineering							*
;*  with assistance from Nautilus Robotics					*
;*  2001.5.15									*
;*										*
;*  Revision History:								*
;*	May  07/01 GP - Create standard code (init, h/w declarations, etc).	*
;*			Add serial transmit and single axis A/D.		*
;*	June 09/01 AK -	Add second A/D axis. Combine A/D data and buttons into 	*
;*			8 bit word for serial transmit.				*
;*	June 17/01 GP - Two byte serial communication and depth control 	*
;*			A/D sampling implemented.				*
;*	June 26/01 AK - Add button handling for servo/light control (BTNTEST 	*
;*			subroutine).						*
;*	June 28/01 AK - Fix bugs in button implmentation.			*
;*	June 30/01 AK - Fix more bugs in button implmentation.			*
;*	July 11/01 GP - Implemented one wire serial communication.		*
;*										*
;*  Pin 7 (AN0) ,analogue input (X plane)					*
;*  Pin 6 (AN1) ,analogue input (Y plane)					*
;*  Pin 5 (AN2) ,analogue input (Depth)						*
;*  Pin 3 (GP4)	,digital input  (button)					*
;*  Pin 2 (GP5) ,digital serial output						*
;*  Pin 4 (GP3) ,reset								*
;*										*
;*  SER_DATA_1 register:							*
;*										*
;*  ___7______6______5______4______3______2______1______0__			*
;* |      |      |      |      |      |      |      |      | 			*
;*  _______________________________________________________			*
;*   LIGHT SERVO   --- LEFT MOTOR ---  --- RIGHT MOTOR ----			*
;* 										*
;*  SER_DATA_2 register:							*
;*										*
;*  ___7______6______5______4______3______2______1______0__			*
;* |      |      |      |      |      |      |      |      | 			*
;*  _______________________________________________________			*
;*  -------------- DEPTH CONTROL A/D VALUE ----------------			*
;*										*
;********************************************************************************


;********************************************************************************
;* Processor definition and configuration					*
;********************************************************************************
; 	Clock = 4MHz internal RC oscillator

  	list	P=12C671	
 	#include "P12C671.inc"

	ERRORLEVEL -302		; turn off bank bits warning

	__CONFIG _CP_OFF & _MCLRE_ON & _WDT_OFF & _INTRC_OSC & _PWRTE_ON


;********************************************************************************
;* Hardware declaration								*
;********************************************************************************

#define	X_AXIS	GPIO, 0			; Pin 7, analogue input
#define	Y_AXIS	GPIO, 1			; Pin 6, analogue input
#define	DEPTH	GPIO, 2			; Pin 5, analogue input
#define	BUTTON	GPIO, 4			; Pin 3, digital input
#define	SEROUT	GPIO, 5			; Pin 2, digital serial output


;********************************************************************************
;* Variable declaration								*
;********************************************************************************

	cblock	0x20

	CNTR1				; temporary counter variable
	CNTR2				; temporary counter variable
 	SER_DATA_1			; 1st byte of serial data to be transmitted
	SER_DATA_2			; 2nd byte of serial data to be transmitted
	SER_LOOP_1			; number of loops to transmit bits of 1st byte
 	SER_LOOP_2			; number of loops to transmit bits of 2nd byte
 	SER_TIME			; delay loop for serial transmitting
	LAST_BTN			; stores previous status of button for compare
	CURRENT_BTN			; stores current status of button for compare
	BTN_COUNT			; loop counter for button handling
	LIGHT_STATUS			; stores light bit value

	endc

;********************************************************************************
;* Constants Declaration							*
;********************************************************************************

BTN_RST	equ	.200			; Loop counter for servo/light button handling


;********************************************************************************
;* Start of Program								*
;********************************************************************************

	org	0x00			
	goto	INIT			; reset vector

	org	0x04
	goto	intr			; interrupt vector

;********************************************************************************
;* Initialize pic								*
;********************************************************************************

INIT	movlw	0x84			; EPROM callibration (for EPROM only)
	movwf	OSCCAL			; Store factory callibration value
	clrf	GPIO			; clear GPIO
	movlw	0x1F			; GPIO, 5 is serial output, the rest input
	tris	GPIO
	movlw	0xC3			; TMR0 enable with 1:16 prescalar
	option

;********************************************************************************
;* Initialize Variables								*
;********************************************************************************
	clrf	CURRENT_BTN		; set button status variable to zero (not pressed)
	clrf	LIGHT_STATUS		; initialize light status to zero (off)
	movlw	BTN_RST
	movwf	BTN_COUNT		; initialize button timer loop to start value


;********************************************************************************
;* Main program loop								*
;********************************************************************************

MAIN	movlw	0xC9			; channel 1 (GP1/AN1/Pin 6), A/D module on
	call	SAMPLE			; sample AN1 (Y axis), value returns in W	
	andlw	0xE0			; mask 3 most sig bits, put result in W
	movwf	SER_DATA_1		; SER_DATA_1 = [y7 y6 y5 0, 0 0 0 0]
	rrf	SER_DATA_1, F
	rrf	SER_DATA_1, F
	rrf	SER_DATA_1, F		; SER_DATA_1 = [0 0 0 y7, y6 y5 0 0]

	movlw	0xC1			; channel 0 (GP0/AN0/Pin 7), A/D module on
	call	SAMPLE			; sample AN0 (X axis), value returns in W
	andlw	0xE0			; mask 3 most sig bits, put result in W
	addwf	SER_DATA_1, F		; SER_DATA_1 = [x7 x6 x5 y7, y6 y5 0 0]
	rrf	SER_DATA_1, F 
	rrf	SER_DATA_1, F		; SER_DATA_1 = [0 0 x7 x6, x5 y7 y6 y5]

	movlw	0xD1			; channel 2 (GP2/AN2/Pin 5), A/D module on
	call	SAMPLE			; sample AN2 (depth), value returns in W
	movwf	SER_DATA_2		; second byte of serial data

	call	BTNTEST			; check button status and adjust light/servo bits
	call 	SERIAL			; transmit SER_DATA_1 and SER_DATA_2

	call	PAUSE			; giver receiver time to syncronize

	goto	MAIN			; main program loop


;********************************************************************************
;* Delay for Reciever to Syncronize						*
;********************************************************************************
; the receiving micro needs to syncronize in order to receive the 16 bit data
; transfer correctly (continuously transmitted). 16 bits takes 832us (52us/bit)
; and the worst case is when all 16 data bits are low. PAUSE is a delay that
; combines with A/D sampling time to create a 832us transmittion silence.
; When the receiving micro detects at least 832us then it can start polling for
; the start bit. This delay is 350us assuming that the main loop takes about 
; 500us to execute.

PAUSE	movlw	.70
	movwf	CNTR1		; outer loop, executed CNTR1 times
LP	nop
	nop
	decfsz	CNTR1,F
	goto	LP

	return

;********************************************************************************
;* Sample A/D channel and store value in W					*
;********************************************************************************

SAMPLE	movwf	ADCON0			; set channel, and turn A/D on
	bsf	STATUS, RP0		; select page 1
	movlw	0x02			; AN0, AN1 & AN2 analogue and GP4 digital
	movwf	ADCON1
	bsf	PIE1, ADIE		; enable A/D interrupt
	bcf	STATUS, RP0		; select page 0

	bcf	PIR1, ADIE		; clear A/D interrupt flag (conversion not complete)
	bsf	INTCON, PEIE		; enable peripheral interrupts
	bsf	INTCON, GIE		; enable all interrupts

	call	DELAY_100us		; allow time for A/D caps to charge (sample time)

	bsf	ADCON0, GO		; start A/D conversion
	sleep				; sleep while conversion takes place

	movf	ADRES, W		; place result in W

	return

;********************************************************************************
;* Interrupt routine								*
;********************************************************************************

intr	clrf	INTCON			; prevent recursive interrupt

	return


;********************************************************************************
;* Serial Output subroutine							*
;********************************************************************************
; Transmit contents of SER_DATA_1 and SER_DATA_2 with start bit, 16 data bits
; and a stop bit. Transmitted at 19.2kbps

SERIAL	movlw 	.9			; 8 data bits and a start bit for the
	movwf	SER_LOOP_1		; first byte
	movlw 	.8			; 8 data bits for second byte
	movwf	SER_LOOP_2
	bsf	STATUS, C		; set C to 1, start bit

OUT_1					; each loop from OUT_1 transmits one bit and 
					; takes 52us/bit (19.2kbps)
	btfsc	STATUS, C		; if 1 (always set on first loop... start bit)...
	bsf	SEROUT			;   ...send a one		
	btfss	STATUS, C		; if not 1...
	bcf	SEROUT			;   ...send a zero

	movlw   .14			; adjust for duration of bit = 52us (@19.2kbps)
        movwf	SER_TIME     		; 

OUT_2	decfsz  SER_TIME, F		; delay loop to account for 42us not used by operations
        goto 	OUT_2			; 
        nop				; nop accounts for timing of final skipped goto
	
	rrf	SER_DATA_1, F		; LSB of SER_DATA_1 now in C
	decfsz	SER_LOOP_1, F		; Decrement loop count (STATUS,C not affected)
	goto	OUT_1			; Start over w/ next bit

					; start of second byte

OUT_3	rrf	SER_DATA_2, F		; LSB of SER_DATA_2 now in C
	btfsc	STATUS, C		; if 1
	bsf	SEROUT			;   ...send a one		
	btfss	STATUS, C		; if not 1...
	bcf	SEROUT			;   ...send a zero

	movlw   .14			; adjust for duration of bit = 52us (@19.2kbps)
        movwf	SER_TIME

OUT_4	decfsz  SER_TIME, F		; delay loop to account for 42us not used by operations
        goto 	OUT_4			; 
        nop				; nop accounts for timing of final skipped goto

	decfsz	SER_LOOP_2, F		; Dec loop count (STATUS,C not affected by dec)
	goto	OUT_3			; Start over w/ next bit

					; end of second byte

	bcf	SEROUT			; send stop bit (low)	
	movlw	.14
	movwf	SER_TIME

OUT_5	decfsz	SER_TIME, F
	goto	OUT_5
	nop

	return


;********************************************************************************
;* Check status of button set light and servo bits and necessary		*
;********************************************************************************

BTNTEST	movf	CURRENT_BTN, W		; place most recent button state in W
	movwf	LAST_BTN 		;  ...and store it in LAST_BTN
	btfsc	LIGHT_STATUS, 0		; check light status variable
	bsf	SER_DATA_1, 7		;  ...and update SER_DATA_1 light bit

BTN_DWN	btfss	BUTTON			; is button pushed?
	goto	BTN_UP			; if not pushed go to BTN_UP
	bsf	CURRENT_BTN, 0		; set CURRENT_BTN status true
	movf	BTN_COUNT, F		; move button loop value into itself
	btfss	STATUS, Z		;  ...and if value is not zero
	decf	BTN_COUNT, F		;  ...then decrement button loop value
	btfsc	STATUS, Z		; if count is zero (or is now zero after dec)...
	bsf	SER_DATA_1, 6		;  ...set servo bit in serial data
	return

BTN_UP	clrf	CURRENT_BTN		; clear CURRENT_BTN status (set to false)
	btfss	LAST_BTN, 0		; check status of button last time
	return				;  ...and if zero then nothing happened so return
	bcf	SER_DATA_1, 6		; button not pushed so clear the servo bit
	movf	BTN_COUNT, F		; move button loop value into itself
	btfss	STATUS, Z		;  ...and if not zero (ie button momentarily pushed)
	call	LIGHT			;  ...then toggle the light
	movlw	BTN_RST			; load reset value for button loop counter
	movwf	BTN_COUNT		;  ... and put it in the loop counter
	return

LIGHT	btfsc	LIGHT_STATUS, 0		; if light bit is set
	goto	LIGHT2			;  ... go to LIGHT2
	bsf	SER_DATA_1, 7		;  ... else if not set, set light bit
	bsf	LIGHT_STATUS, 0		;  ... and set light status variable
	return				

LIGHT2	bcf	SER_DATA_1, 7		; light bit is set, clear it
	clrf	LIGHT_STATUS		; clear light status variable
	return


;********************************************************************************
;* Delay subroutine - delays W milleconds (load W with delay time)		*
;********************************************************************************

DELAY	movwf	CNTR2
	
LOOP_B	movlw	.200		; outer loop, executed CNTR2 times
	movwf	CNTR1

LOOP_A	nop			; inner loop, executed CNTR1 times
	nop
	decfsz	CNTR1,F
	goto	LOOP_A

	decfsz	CNTR2,F		
	goto	LOOP_B	

	return


;********************************************************************************
;* Sample time delay subroutine - 100us						*
;********************************************************************************

DELAY_100us

	movlw	.20
	movwf	CNTR1		; outer loop, executed CNTR1 times
LOOP	nop
	nop
	decfsz	CNTR1,F
	goto	LOOP

	return


;********************************************************************************
;* End of program								*
;********************************************************************************

	end


